@@ -20,7 +20,7 @@ import io
 import six
 from six.moves import xrange      # pylint: disable=redefined-builtin
 from .line_numbers import LineNumbers
-from .util import Token, match_token
+from .util import Token, match_token, is_non_coding_token
 from .mark_tokens import MarkTokens
 
 class ASTTokens(object):
@@ -43,12 +43,15 @@ class ASTTokens(object):
   tree created separately.
   """
   def __init__(self, source_text, parse=False, tree=None, filename='<unknown>'):
-    if isinstance(source_text, six.binary_type):
-      source_text = source_text.decode('utf8')
-
     self._filename = filename
     self._tree = ast.parse(source_text, filename) if parse else tree
 
+    # Decode source after parsing to let Python 2 handle coding declarations.
+    # (If the encoding was not utf-8 compatible, then even if it parses correctly,
+    # we'll fail with a unicode error here.)
+    if isinstance(source_text, six.binary_type):
+      source_text = source_text.decode('utf8')
+
     self._text = source_text
     self._line_numbers = LineNumbers(source_text)
 
@@ -135,7 +138,7 @@ class ASTTokens(object):
     """
     i = tok.index + 1
     if not include_extra:
-      while self._tokens[i].type >= token.N_TOKENS:
+      while is_non_coding_token(self._tokens[i].type):
         i += 1
     return self._tokens[i]
 
@@ -146,7 +149,7 @@ class ASTTokens(object):
     """
     i = tok.index - 1
     if not include_extra:
-      while self._tokens[i].type >= token.N_TOKENS:
+      while is_non_coding_token(self._tokens[i].type):
         i -= 1
     return self._tokens[i]
 
@@ -168,7 +171,7 @@ class ASTTokens(object):
     include_extra is True, includes non-coding tokens such as tokenize.NL and .COMMENT.
     """
     for i in xrange(first_token.index, last_token.index + 1):
-      if include_extra or self._tokens[i].type < token.N_TOKENS:
+      if include_extra or not is_non_coding_token(self._tokens[i].type):
         yield self._tokens[i]
 
   def get_tokens(self, node, include_extra=False):
